﻿using UnityEngine;
using System.Collections.Generic;
using Newtonsoft.Json;

/// <summary>
/// Loads and processes CO2 emissions data from a JSON file.
/// Deserializes the data and passes it to the DataVisualizer for visualization.
/// </summary>
public class DataLoader : MonoBehaviour
{
    /// <summary>
    /// Reference to the DataVisualizer for creating visualizations.
    /// </summary>
    public DataVisualizer Visualizer;

    /// <summary>
    /// Unity Start method, called before the first frame update.
    /// Loads JSON data, deserializes it, and initializes visualizations.
    /// </summary>
    void Start()
    {
        TextAsset jsonData = Resources.Load<TextAsset>("co2-year-country");
        string json = jsonData.text;

        // Use custom deserialization to handle the hierarchical structure
        CO2Data data = DeserializeCO2Data(json);

        // Pass the deserialized data to the visualizer
        Visualizer.CreateMeshes(data.Years);
    }

    /// <summary>
    /// Deserializes JSON data into a CO2Data object.
    /// </summary>
    /// <param name="json">The JSON string to deserialize.</param>
    /// <returns>A CO2Data object containing the deserialized data.</returns>
    private CO2Data DeserializeCO2Data(string json)
    {
        return JsonConvert.DeserializeObject<CO2Data>(json);
    }
}

/// <summary>
/// Represents the top-level structure of CO2 emissions data.
/// </summary>
[System.Serializable]
public class CO2Data
{
    /// <summary>
    /// A list of yearly data entries.
    /// </summary>
    public List<YearData> Years;
}

/// <summary>
/// Represents CO2 data for a specific year.
/// </summary>
[System.Serializable]
public class YearData
{
    /// <summary>
    /// The year of the data entry.
    /// </summary>
    public string Year;

    /// <summary>
    /// Dictionary mapping country names to their respective data.
    /// </summary>
    public Dictionary<string, CountryData> Countries;
}

/// <summary>
/// Represents CO2 data for a specific country.
/// </summary>
[System.Serializable]
public class CountryData
{
    /// <summary>
    /// The latitude of the country.
    /// </summary>
    public float Latitude;

    /// <summary>
    /// The longitude of the country.
    /// </summary>
    public float Longitude;

    /// <summary>
    /// Total CO2 emissions of the country.
    /// </summary>
    public float Co2;

    /// <summary>
    /// The population of the country.
    /// </summary>
    public float Population;

    /// <summary>
    /// The area of the country.
    /// </summary>
    public float Area;

    /// <summary>
    /// CO2 emissions per capita.
    /// </summary>
    public float Co2PerCapita;
}