﻿using UnityEngine;
using System.Collections.Generic;
using System;
using Unity.VisualScripting;
using UnityEngine.UIElements;
using System.Collections;

/// <summary>
/// Visualizes CO2 emissions data on a globe.
/// Handles the creation and management of meshes and UI updates for CO2 data visualization.
/// </summary>
public class DataVisualizer : MonoBehaviour
{
    /// <summary>
    /// Material for absolute CO2 emission bars.
    /// </summary>
    public Material pointMaterial;

    /// <summary>
    /// Material for per capita CO2 emission bars.
    /// </summary>
    public Material pointMaterialCap;

    /// <summary>
    /// Gradient for absolute CO2 emissions.
    /// </summary>
    public Gradient colorsAbs;

    /// <summary>
    /// Gradient for per capita CO2 emissions.
    /// </summary>
    public Gradient colorsCap;

    /// <summary>
    /// Reference to the globe GameObject.
    /// </summary>
    public GameObject earth;

    /// <summary>
    /// Prefab for data point representation.
    /// </summary>
    public GameObject pointPrefab;

    /// <summary>
    /// Reference to the UIHandler for UI updates.
    /// </summary>
    public UIHandler uiHandler;

    /// <summary>
    /// Reference to the DetailsOnDemand component for showing detailed information.
    /// </summary>
    public DetailsOnDemand DetailsOnDemand;

    private Dictionary<string, GameObject> countryObjs = new Dictionary<string, GameObject>();
    private List<YearData> data = new List<YearData>();

    private int currentYear = 2020;
    private Coroutine autoPlayCoroutine;

    private const float MAX_CO2 = 236000000000.0f;
    private const float MAX_CAP = 1298.63f;

    private float worldPopSum;
    private float co2Sum;

    /// <summary>
    /// Gets the scale of the globe.
    /// </summary>
    /// <returns>The scale of the globe as a Vector3.</returns>
    public Vector3 GetGlobeScale()
    {
        return earth.transform.localScale;
    }

    /// <summary>
    /// Creates visual meshes for the given years of CO2 data.
    /// </summary>
    /// <param name="years">List of YearData objects to visualize.</param>
    public void CreateMeshes(List<YearData> years)
    {
        data = years;

        YearData yearData = years[years.Count - 1];
        foreach (var countryEntry in yearData.Countries)
        {
            string countryName = countryEntry.Key;
            CountryData countryData = countryEntry.Value;

            CreateCountryObject(countryName, countryData);
        }

        UpdateUIInfo();
    }

    /// <summary>
    /// Creates a GameObject to represent CO2 data for a specific country.
    /// </summary>
    /// <param name="countryName">The name of the country.</param>
    /// <param name="countryData">The CO2 data of the country.</param>
    private void CreateCountryObject(string countryName, CountryData countryData)
    {
        GameObject parentObj = new GameObject(countryName);
        parentObj.transform.parent = earth.transform;
        parentObj.transform.localPosition = Vector3.zero;

        Vector3 basePosition = CalculatePositionOnEarth(countryData.Latitude, countryData.Longitude);
        Vector3 upDirection = basePosition.normalized;
        Vector3 offsetDirection = Vector3.Cross(upDirection, Vector3.up).normalized * 0.005f;

        GameObject countryObject = Instantiate(pointPrefab, parentObj.transform);
        countryObject.name = countryName + "Abs";
        countryObject.transform.parent = parentObj.transform;
        countryObject.transform.localPosition = basePosition + offsetDirection;
        countryObject.transform.localScale = new Vector3(3, 3, Mathf.Max(0.001f, countryData.Co2 / MAX_CO2));
        countryObject.transform.LookAt(countryObject.transform.localPosition * 2 + earth.transform.position);

        GameObject capitaObject = Instantiate(pointPrefab, parentObj.transform);
        capitaObject.name = countryName + "Cap";
        capitaObject.transform.parent = parentObj.transform;
        capitaObject.transform.localPosition = basePosition - offsetDirection;
        capitaObject.transform.localScale = new Vector3(3, 3, Mathf.Max(0.001f, countryData.Co2PerCapita / MAX_CAP));
        capitaObject.transform.LookAt(capitaObject.transform.localPosition * 2 + earth.transform.position);

        ApplyMeshColor(countryObject, colorsAbs.Evaluate(Mathf.Max(0.001f, countryData.Co2 / MAX_CO2)), pointMaterial);
        ApplyMeshColor(capitaObject, colorsCap.Evaluate(Mathf.Max(0.001f, countryData.Co2PerCapita / MAX_CAP)), pointMaterialCap);

        worldPopSum += countryData.Population;
        co2Sum += countryData.Co2;

        countryObjs.Add(countryName, parentObj);
    }

    /// <summary>
    /// Applies color and material to a mesh.
    /// </summary>
    /// <param name="obj">The GameObject representing the mesh.</param>
    /// <param name="color">The color to apply.</param>
    /// <param name="material">The material to use.</param>
    private void ApplyMeshColor(GameObject obj, Color color, Material material)
    {
        Mesh mesh = obj.GetComponent<MeshFilter>().mesh;
        List<Color> meshColors = new List<Color>();
        Vector3[] verts = mesh.vertices;

        for (int k = 0; k < verts.Length; k++)
        {
            meshColors.Add(color);
        }

        mesh.colors = meshColors.ToArray();
        var renderer = obj.GetComponent<MeshRenderer>();
        renderer.material = material;
    }

    /// <summary>
    /// Calculates the position of a point on the Earth's surface.
    /// </summary>
    /// <param name="lat">The latitude of the point.</param>
    /// <param name="lng">The longitude of the point.</param>
    /// <returns>A Vector3 representing the position on the Earth's surface.</returns>
    private Vector3 CalculatePositionOnEarth(float lat, float lng)
    {
        float radius = earth.GetComponent<SphereCollider>().radius;
        Vector3 position;
        position.x = radius * Mathf.Cos(lat * Mathf.Deg2Rad) * Mathf.Cos(lng * Mathf.Deg2Rad);
        position.y = radius * Mathf.Sin(lat * Mathf.Deg2Rad);
        position.z = radius * Mathf.Cos(lat * Mathf.Deg2Rad) * Mathf.Sin(lng * Mathf.Deg2Rad);
        return position;
    }

    /// <summary>
    /// Changes the displayed year and updates the data visualization.
    /// </summary>
    /// <param name="year">The year to change to.</param>
    public void ChangeYear(System.Single year)
    {
        currentYear = (int)year;
        co2Sum = 0;
        worldPopSum = 0;

        YearData yearData = data.Find(y => y.Year == year.ToString());
        if (yearData == null)
        {
            Debug.LogWarning($"Year {currentYear} not found in data.");
            return;
        }

        foreach (var countryEntry in yearData.Countries)
        {
            string countryName = countryEntry.Key;
            CountryData countryData = countryEntry.Value;

            if (countryObjs.TryGetValue(countryName, out GameObject parentObj))
            {
                Transform absBar = parentObj.transform.Find(countryName + "Abs");
                if (absBar != null)
                {
                    absBar.localScale = new Vector3(3, 3, Mathf.Max(0.001f, countryData.Co2 / MAX_CO2));
                    ApplyMeshColor(absBar.gameObject, colorsAbs.Evaluate(Mathf.Max(0.001f, countryData.Co2 / MAX_CO2)), pointMaterial);
                }

                Transform capBar = parentObj.transform.Find(countryName + "Cap");
                if (capBar != null)
                {
                    capBar.localScale = new Vector3(3, 3, Mathf.Max(0.001f, countryData.Co2PerCapita / MAX_CAP));
                    ApplyMeshColor(capBar.gameObject, colorsCap.Evaluate(Mathf.Max(0.001f, countryData.Co2PerCapita / MAX_CAP)), pointMaterialCap);
                }

                worldPopSum += countryData.Population;
                co2Sum += countryData.Co2;
            }
            else
            {
                Debug.LogWarning($"Country {countryName} not found in country objects.");
            }
        }

        UpdateUIInfo();
    }

    /// <summary>
    /// Starts autoplay for changing years.
    /// </summary>
    public void AutoPlayYears()
    {
        if (autoPlayCoroutine != null)
        {
            StopCoroutine(autoPlayCoroutine);
        }
        autoPlayCoroutine = StartCoroutine(AutoPlayYearsCoroutine());
    }

    /// <summary>
    /// Coroutine for autoplaying years in sequence.
    /// </summary>
    private IEnumerator AutoPlayYearsCoroutine()
    {
        int startYear = currentYear;
        int endYear = Convert.ToInt32(data[data.Count - 1].Year);

        if (startYear == endYear)
        {
            startYear = Convert.ToInt32(data[0].Year);
        }

        uiHandler.DeactivateSlider();

        for (int year = startYear; year <= endYear; year++)
        {
            ChangeYear(year);
            uiHandler.ChangeYear(year);
            yield return new WaitForSeconds(0.3f);
        }

        uiHandler.ActivateSlider();
    }

    /// <summary>
    /// Toggles the visibility of absolute CO2 emission bars.
    /// </summary>
    /// <param name="isActive">Whether to activate or deactivate the bars.</param>
    public void ToggleAbs(System.Boolean isActive)
    {
        foreach (var countryObj in countryObjs)
        {
            Transform absBar = countryObj.Value.transform.Find(countryObj.Key + "Abs");
            if (absBar != null)
            {
                absBar.gameObject.SetActive(isActive);
            }
        }
    }

    /// <summary>
    /// Toggles the visibility of per capita CO2 emission bars.
    /// </summary>
    /// <param name="isActive">Whether to activate or deactivate the bars.</param>
    public void ToggleCap(System.Boolean isActive)
    {
        foreach (var countryObj in countryObjs)
        {
            Transform capBar = countryObj.Value.transform.Find(countryObj.Key + "Cap");
            if (capBar != null)
            {
                capBar.gameObject.SetActive(isActive);
            }
        }
    }

    /// <summary>
    /// Displays detailed information about a country.
    /// </summary>
    /// <param name="name">The name of the country.</param>
    /// <param name="position">The position to display the details at.</param>
    public void ShowDetailsOnDemand(String name, Vector3 position)
    {
        YearData yearData = data.Find(y => y.Year == currentYear.ToString());
        CountryData cData = yearData.Countries[name];
        DetailsOnDemand.ShowDetails(name, cData.Co2, cData.Co2PerCapita, cData.Population, cData.Area, position);
    }

    /// <summary>
    /// Removes any displayed detailed information.
    /// </summary>
    public void RemoveDetailsOnDemand()
    {
        DetailsOnDemand.RemoveDetails();
    }

    /// <summary>
    /// Updates the UI with current CO2 and population data.
    /// </summary>
    private void UpdateUIInfo()
    {
        uiHandler.ChangeCo2Sum(co2Sum.ToString());
        uiHandler.ChangeWorldPopulation(worldPopSum.ToString());
        uiHandler.ChangePerCapSum((co2Sum / worldPopSum).ToString());
    }
}
