using TMPro;
using UnityEngine;

/// <summary>
/// Manages the display of detailed information about a specific country.
/// Handles the positioning and updating of the detail display.
/// </summary>
public class DetailsOnDemand : MonoBehaviour
{
    /// <summary>
    /// The GameObject containing the detail UI.
    /// </summary>
    [SerializeField] private GameObject details;

    /// <summary>
    /// Text field for displaying the country name.
    /// </summary>
    [SerializeField] private TMP_Text country;

    /// <summary>
    /// Text field for displaying CO2 emissions.
    /// </summary>
    [SerializeField] private TMP_Text co2;

    /// <summary>
    /// Text field for displaying per capita CO2 emissions.
    /// </summary>
    [SerializeField] private TMP_Text perCap;

    /// <summary>
    /// Text field for displaying the population.
    /// </summary>
    [SerializeField] private TMP_Text population;

    /// <summary>
    /// Text field for displaying the area.
    /// </summary>
    [SerializeField] private TMP_Text area;

    /// <summary>
    /// Updates the rotation of the details panel to face the camera.
    /// </summary>
    void LateUpdate()
    {
        details.transform.LookAt(Camera.main.transform.position, Camera.main.transform.up);
    }

    /// <summary>
    /// Displays the details with the provided data.
    /// </summary>
    /// <param name="name">The name of the country.</param>
    /// <param name="co2">The CO2 emissions of the country.</param>
    /// <param name="perCap">The per capita CO2 emissions of the country.</param>
    /// <param name="population">The population of the country.</param>
    /// <param name="area">The area of the country.</param>
    /// <param name="position">The position to display the details at.</param>
    public void ShowDetails(string name, float co2, float perCap, float population, float area, Vector3 position)
    {
        SetCountry(name);
        SetCo2(co2);
        SetPerCap(perCap);
        SetPopulation(population);
        SetGlobalPosition(position);
        SetArea(area);
        ShowDetails();
    }

    /// <summary>
    /// Activates the details panel.
    /// </summary>
    public void ShowDetails()
    {
        details.SetActive(true);
    }

    /// <summary>
    /// Hides the details panel.
    /// </summary>
    public void RemoveDetails()
    {
        details.SetActive(false);
    }

    /// <summary>
    /// Updates the displayed country name.
    /// </summary>
    /// <param name="name">The name of the country.</param>
    public void SetCountry(string name)
    {
        country.text = name;
    }

    /// <summary>
    /// Updates the displayed CO2 emissions.
    /// </summary>
    /// <param name="data">The CO2 emissions value.</param>
    public void SetCo2(float data)
    {
        co2.text = data.ToString();
    }

    /// <summary>
    /// Updates the displayed per capita CO2 emissions.
    /// </summary>
    /// <param name="data">The per capita CO2 emissions value.</param>
    public void SetPerCap(float data)
    {
        perCap.text = data.ToString();
    }

    /// <summary>
    /// Updates the displayed population.
    /// </summary>
    /// <param name="data">The population value.</param>
    public void SetPopulation(float data)
    {
        population.text = data.ToString();
    }

    /// <summary>
    /// Updates the displayed area.
    /// </summary>
    /// <param name="data">The area value.</param>
    public void SetArea(float data)
    {
        area.text = data.ToString();
    }

    /// <summary>
    /// Updates the global position of the details panel.
    /// </summary>
    /// <param name="position">The new global position.</param>
    public void SetGlobalPosition(Vector3 position)
    {
        details.transform.position = position;
    }
}
