using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.XR;
using UnityEngine.XR.Interaction.Toolkit;
using UnityEngine.XR.Interaction.Toolkit.Interactables;
using UnityEngine.XR.Interaction.Toolkit.Interactors;

/// <summary>
/// Handles interactions with the globe in a VR environment.
/// Provides functionality for rotating and resizing the globe.
/// </summary>
public class GlobeInteraction : XRBaseInteractable
{
    [Header("Rotation Settings")]
    /// <summary>
    /// Reference to the globe transform.
    /// </summary>
    [SerializeField] private Transform globe;

    /// <summary>
    /// Speed at which the globe rotates.
    /// </summary>
    public float rotationSpeed = 100f;

    [Header("Resizing Settings")]
    /// <summary>
    /// Speed at which the globe is resized.
    /// </summary>
    public float resizeSpeed = 0.2f;

    /// <summary>
    /// Minimum allowed scale for the globe.
    /// </summary>
    public float minScale = 0.5f;

    /// <summary>
    /// Maximum allowed scale for the globe.
    /// </summary>
    public float maxScale = 2f;

    private bool isResizing = false;
    private float initialDistance = 0f;
    private Vector3 initialScale;
    private Vector3 initialGlobePosition;
    private Vector3 midPointOffset;

    private IXRSelectInteractor firstController;
    private IXRSelectInteractor secondController;

    private IXRSelectInteractable interactable;
    private Vector3 previousControllerPosition;

    /// <summary>
    /// Updates the interaction state. Handles resizing or rotation based on the current state.
    /// </summary>
    private void Update()
    {
        if (isResizing && firstController != null && secondController != null)
        {
            ResizeAndMoveGlobe();
        }
        else if (firstController != null && globe != null)
        {
            RotateGlobe();
        }
    }

    /// <summary>
    /// Handles the event when an interactor starts interacting with the globe.
    /// </summary>
    /// <param name="args">Event arguments containing interactor and interactable details.</param>
    protected override void OnSelectEntered(SelectEnterEventArgs args)
    {
        base.OnSelectEntered(args);

        if (HasMultipleInteractors())
        {
            secondController = args.interactorObject;
            StartResizingAndMoving();
        }
        else
        {
            firstController = args.interactorObject;
            interactable = args.interactableObject;
            previousControllerPosition = firstController.transform.position;
        }
    }

    /// <summary>
    /// Checks if multiple interactors are currently selecting the globe.
    /// </summary>
    /// <returns>True if multiple interactors are present, otherwise false.</returns>
    private bool HasMultipleInteractors()
    {
        return interactorsSelecting.Count > 1;
    }

    /// <summary>
    /// Handles the event when an interactor stops interacting with the globe.
    /// </summary>
    /// <param name="args">Event arguments containing interactor and interactable details.</param>
    protected override void OnSelectExited(SelectExitEventArgs args)
    {
        base.OnSelectExited(args);
        firstController = null;
        secondController = null;
        interactable = null;
        isResizing = false;
    }

    /// <summary>
    /// Initializes the resizing and moving state for the globe.
    /// </summary>
    public void StartResizingAndMoving()
    {
        initialDistance = Vector3.Distance(firstController.transform.position, secondController.transform.position);
        initialScale = globe.localScale;
        initialGlobePosition = globe.position;
        midPointOffset = initialGlobePosition - GetMidPointBetweenControllers();
        isResizing = true;
    }

    /// <summary>
    /// Handles resizing and repositioning the globe based on controller movements.
    /// </summary>
    private void ResizeAndMoveGlobe()
    {
        float currentDistance = Vector3.Distance(firstController.transform.position, secondController.transform.position);
        float scaleChange = (currentDistance - initialDistance) * resizeSpeed;

        Vector3 newScale = initialScale + Vector3.one * scaleChange;
        newScale.x = Mathf.Clamp(newScale.x, minScale, maxScale);
        newScale.y = Mathf.Clamp(newScale.y, minScale, maxScale);
        newScale.z = Mathf.Clamp(newScale.z, minScale, maxScale);

        globe.localScale = newScale;

        Vector3 midPoint = GetMidPointBetweenControllers();
        globe.position = midPoint + midPointOffset;
    }

    /// <summary>
    /// Calculates the midpoint between two controllers.
    /// </summary>
    /// <returns>The midpoint position as a Vector3.</returns>
    private Vector3 GetMidPointBetweenControllers()
    {
        return (firstController.transform.position + secondController.transform.position) / 2f;
    }

    /// <summary>
    /// Rotates the globe based on the movement of the controller.
    /// </summary>
    private void RotateGlobe()
    {
        Vector3 currentControllerPosition = firstController.transform.position;
        Vector3 movement = currentControllerPosition - previousControllerPosition;

        Vector3 movementOnPlane = new Vector3(movement.x, 0, movement.z);
        float rotationAmount = movementOnPlane.x * rotationSpeed;

        globe.Rotate(Vector3.up, -rotationAmount, Space.World);

        previousControllerPosition = currentControllerPosition;
    }
}
