using TMPro;
using UnityEngine;

/// <summary>
/// Handles the user interface (UI) for displaying and updating year, population, and CO2 data.
/// </summary>
public class UIHandler : MonoBehaviour
{
    /// <summary>
    /// Text field for displaying the current year.
    /// </summary>
    [SerializeField] private TMP_Text year;

    /// <summary>
    /// Slider component for year selection.
    /// </summary>
    [SerializeField] private UnityEngine.UI.Slider slider;

    /// <summary>
    /// Text field for displaying the world population.
    /// </summary>
    [SerializeField] private TMP_Text worldPop;

    /// <summary>
    /// Text field for displaying the total CO2 emissions.
    /// </summary>
    [SerializeField] private TMP_Text worldCo2;

    /// <summary>
    /// Text field for displaying the per capita CO2 emissions sum.
    /// </summary>
    [SerializeField] private TMP_Text perCapSum;

    /// <summary>
    /// Updates the year display based on the slider value.
    /// </summary>
    /// <param name="value">The slider value representing the year.</param>
    public void SliderChange(System.Single value)
    {
        year.text = ((int)value).ToString();
    }

    /// <summary>
    /// Changes the displayed year and updates the slider value.
    /// </summary>
    /// <param name="newYear">The new year to display.</param>
    public void ChangeYear(int newYear)
    {
        year.text = newYear.ToString();
        slider.value = newYear;
    }

    /// <summary>
    /// Disables the slider component.
    /// </summary>
    public void DeactivateSlider()
    {
        slider.enabled = false;
    }

    /// <summary>
    /// Enables the slider component.
    /// </summary>
    public void ActivateSlider()
    {
        slider.enabled = true;
    }

    /// <summary>
    /// Updates the displayed world population.
    /// </summary>
    /// <param name="population">The population value as a string.</param>
    public void ChangeWorldPopulation(string population)
    {
        worldPop.text = population;
    }

    /// <summary>
    /// Updates the displayed total CO2 emissions.
    /// </summary>
    /// <param name="co2sum">The total CO2 emissions value as a string.</param>
    public void ChangeCo2Sum(string co2sum)
    {
        worldCo2.text = co2sum;
    }

    /// <summary>
    /// Updates the displayed per capita CO2 emissions sum.
    /// </summary>
    /// <param name="perCap">The per capita CO2 value as a string.</param>
    public void ChangePerCapSum(string perCap)
    {
        perCapSum.text = perCap;
    }
}
